@@ -1,4 +1,4 @@
-/*	$OpenBSD: output.c,v 1.59 2025/01/29 13:14:41 claudio Exp $ */
+/*	$OpenBSD: output.c,v 1.67 2025/12/29 07:49:05 claudio Exp $ */
 
 /*
  * Copyright (c) 2003 Henning Brauer <henning@openbsd.org>
@@ -19,6 +19,9 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <sys/socket.h>
+#include <arpa/inet.h>
+
 #include <endian.h>
 #include <err.h>
 #include <math.h>
@@ -314,7 +317,7 @@ show_neighbor_full(struct peer *p, struct parse_result *res)
 		printf(" with shutdown reason \"%s\"",
 		    log_reason(p->conf.reason));
 	}
-	if (p->stats.last_updown != 0)
+	if (monotime_valid(p->stats.last_updown))
 		printf(", %s for %s",
 		    p->state == STATE_ESTABLISHED ? "up" : "down",
 		    fmt_monotime(p->stats.last_updown));
@@ -467,10 +470,10 @@ show_timer(struct ctl_timer *t)
 {
 	printf("  %-20s ", timernames[t->type]);
 
-	if (t->val <= 0)
-		printf("%-20s\n", "due");
+	if (get_rel_monotime(t->val) >= 0)
+		printf("%s\n", "due");
 	else
-		printf("due in %-13s\n", fmt_timeframe(t->val));
+		printf("%s\n", fmt_monotime(t->val));
 }
 
 static void
@@ -1033,7 +1036,7 @@ show_rib_detail(struct ctl_show_rib *r, struct ibuf *asbuf, int flag0)
 	    fmt_flags(r->flags, 0));
 
 	printf("%c    Last update: %s ago%c", EOL0(flag0),
-	    fmt_timeframe(r->age), EOL0(flag0));
+	    fmt_monotime(r->lastchange), EOL0(flag0));
 }
 
 static void
@@ -1066,6 +1069,15 @@ show_rib_mem(struct rde_memstats *stats)
 	printf("%10lld prefix entries using %s of memory\n",
 	    stats->prefix_cnt, fmt_mem(stats->prefix_cnt *
 	    sizeof(struct prefix)));
+	printf("%10lld adjout_prefix entries using %s out of",
+	    stats->adjout_prefix_cnt,
+	    fmt_mem(stats->adjout_prefix_cnt * sizeof(struct adjout_prefix)));
+	printf(" %s memory\n", fmt_mem(stats->adjout_prefix_size));
+	printf("%10lld adjout attribute entries using %s of memory\n",
+	    stats->adjout_attr_cnt, fmt_mem(stats->adjout_attr_cnt *
+	    sizeof(struct adjout_attr)));
+	printf("\t   and holding %lld references\n",
+	    stats->adjout_attr_refs);
 	printf("%10lld BGP path attribute entries using %s of memory\n",
 	    stats->path_cnt, fmt_mem(stats->path_cnt *
 	    sizeof(struct rde_aspath)));
@@ -1086,6 +1098,12 @@ show_rib_mem(struct rde_memstats *stats)
 	    stats->attr_refs);
 	printf("%10lld BGP attributes using %s of memory\n",
 	    stats->attr_dcnt, fmt_mem(stats->attr_data));
+	printf("%10lld pending attribute entries using %s of memory\n",
+	    stats->pend_attr_cnt, fmt_mem(stats->pend_attr_cnt *
+	    sizeof(struct pend_attr)));
+	printf("%10lld pending prefix entries using %s of memory\n",
+	    stats->pend_prefix_cnt, fmt_mem(stats->pend_prefix_cnt *
+	    sizeof(struct pend_prefix)));
 	printf("%10lld as-set elements in %lld tables using "
 	    "%s of memory\n", stats->aset_nmemb, stats->aset_cnt,
 	    fmt_mem(stats->aset_size));
@@ -1093,12 +1111,28 @@ show_rib_mem(struct rde_memstats *stats)
 	    stats->pset_cnt, fmt_mem(stats->pset_size));
 	printf("RIB using %s of memory\n", fmt_mem(pts +
 	    stats->prefix_cnt * sizeof(struct prefix) +
+	    stats->adjout_prefix_cnt * sizeof(struct adjout_prefix) +
+	    stats->adjout_attr_cnt * sizeof(struct adjout_attr) +
+	    stats->pend_prefix_cnt * sizeof(struct pend_prefix) +
+	    stats->pend_attr_cnt * sizeof(struct pend_attr) +
 	    stats->rib_cnt * sizeof(struct rib_entry) +
 	    stats->path_cnt * sizeof(struct rde_aspath) +
 	    stats->aspath_size + stats->attr_cnt * sizeof(struct attr) +
 	    stats->attr_data));
 	printf("Sets using %s of memory\n", fmt_mem(stats->aset_size +
 	    stats->pset_size));
+
+	printf("\nRDE timing statistics\n");
+	printf("%10lld usec spent in the event loop for %llu rounds\n",
+	    stats->rde_event_loop_usec, stats->rde_event_loop_count);
+	printf("%10lld usec spent on io\n", stats->rde_event_io_usec);
+	printf("%10lld usec spent on peers\n", stats->rde_event_peer_usec);
+	printf("%10lld usec spent on adj-out\n", stats->rde_event_adjout_usec);
+	printf("%10lld usec spent on rib dumps\n",
+	    stats->rde_event_ribdump_usec);
+	printf("%10lld usec spent on nexthops\n",
+	    stats->rde_event_nexthop_usec);
+	printf("%10lld usec spent on updates\n", stats->rde_event_update_usec);
 }
 
 static void
@@ -1113,7 +1147,7 @@ show_rib_set(struct ctl_show_set *set)
 		snprintf(buf, sizeof(buf), "%7zu %7zu %6s",
 		    set->v4_cnt, set->v6_cnt, "-");
 
-	printf("%-6s %-34s %s %11s\n", fmt_set_type(set), set->name,
+	printf("%-6s %-34s %s %12s\n", fmt_set_type(set), set->name,
 	    buf, fmt_monotime(set->lastchange));
 }
 
