@@ -1,4 +1,4 @@
-/*	$OpenBSD: session.h,v 1.185 2025/01/31 20:07:18 claudio Exp $ */
+/*	$OpenBSD: session.h,v 1.193 2025/11/04 10:47:25 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -144,9 +144,9 @@ struct peer_stats {
 	unsigned long long	 prefix_sent_update;
 	unsigned long long	 prefix_sent_withdraw;
 	unsigned long long	 prefix_sent_eor;
-	time_t			 last_updown;
-	time_t			 last_read;
-	time_t			 last_write;
+	monotime_t		 last_updown;
+	monotime_t		 last_read;
+	monotime_t		 last_write;
 	uint32_t		 msg_queue_len;
 	uint32_t		 prefix_cnt;
 	uint32_t		 prefix_out_cnt;
@@ -183,17 +183,16 @@ enum Timer {
 	Timer_Rtr_Retry,
 	Timer_Rtr_Expire,
 	Timer_Rtr_Active,
+	Timer_Mrt_Reopen,
 	Timer_Max
 };
 
 struct timer {
 	TAILQ_ENTRY(timer)	entry;
 	enum Timer		type;
-	time_t			val;
+	monotime_t		val;
 };
 
-TAILQ_HEAD(timer_head, timer);
-
 struct peer {
 	struct peer_config	 conf;
 	struct peer_stats	 stats;
@@ -216,6 +215,7 @@ struct peer {
 	u_int			 errcnt;
 	u_int			 IdleHoldTime;
 	unsigned int		 if_scope;	/* interface scope for IPv6 */
+	uint32_t		 local_bgpid;
 	uint32_t		 remote_bgpid;
 	enum session_state	 state;
 	enum session_state	 prev_state;
@@ -233,11 +233,11 @@ struct peer {
 	uint8_t			 rdesession;
 };
 
-extern time_t		 pauseaccept;
+extern monotime_t		 pauseaccept;
 
 struct ctl_timer {
 	enum Timer	type;
-	time_t		val;
+	monotime_t	val;
 };
 
 /* carp.c */
@@ -262,7 +262,7 @@ unsigned int	control_accept(int, int);
 
 /* log.c */
 char	*log_fmt_peer(const struct peer_config *);
-void	 log_statechange(struct peer *, enum session_state,
+void	 log_statechange(struct peer *,  enum session_state,
 	    enum session_events);
 void	 log_notification(const struct peer *, uint8_t, uint8_t,
 	    const struct ibuf *, const char *);
@@ -272,8 +272,7 @@ void	 log_conn_attempt(const struct peer *, struct sockaddr *,
 /* mrt.c */
 void	 mrt_dump_bgp_msg(struct mrt *, struct ibuf *, struct peer *,
 	    enum msg_type);
-void	 mrt_dump_state(struct mrt *, uint16_t, uint16_t,
-	    struct peer *);
+void	 mrt_dump_state(struct mrt *, struct peer *);
 void	 mrt_done(struct mrt *);
 
 /* pfkey.c */
@@ -301,7 +300,7 @@ void	rde_main(int, int);
 struct rtr_session;
 size_t			 rtr_count(void);
 void			 rtr_check_events(struct pollfd *, size_t);
-size_t			 rtr_poll_events(struct pollfd *, size_t, time_t *);
+size_t			 rtr_poll_events(struct pollfd *, size_t, monotime_t *);
 struct rtr_session	*rtr_new(uint32_t, struct rtr_config_msg *);
 struct rtr_session	*rtr_get(uint32_t);
 void			 rtr_free(struct rtr_session *);
@@ -328,23 +327,54 @@ void	rtr_recalc(void);
 RB_PROTOTYPE(peer_head, peer, entry, peer_compare);
 
 void		 session_main(int, int);
-void		 bgp_fsm(struct peer *, enum session_events, struct ibuf *);
 int		 session_neighbor_rrefresh(struct peer *p);
+void		 get_alternate_addr(struct bgpd_addr *, struct bgpd_addr *,
+		    struct bgpd_addr *, unsigned int *);
 struct peer	*getpeerbydesc(struct bgpd_config *, const char *);
 struct peer	*getpeerbyip(struct bgpd_config *, struct sockaddr *);
 struct peer	*getpeerbyid(struct bgpd_config *, uint32_t);
+void		 session_handle_update(struct peer *, struct ibuf *);
+void		 session_handle_rrefresh(struct peer *, struct route_refresh *);
+void		 session_graceful_restart(struct peer *);
+void		 session_graceful_flush(struct peer *, uint8_t, const char *);
+void		 session_mrt_dump_state(struct peer *);
+void		 session_mrt_dump_bgp_msg(struct peer *, struct ibuf *,
+		    enum msg_type, enum directions);
 int		 peer_matched(struct peer *, struct ctl_neighbor *);
 int		 imsg_ctl_parent(struct imsg *);
 int		 imsg_ctl_rde(struct imsg *);
 int		 imsg_ctl_rde_msg(int, uint32_t, pid_t);
+int		 session_connect(struct peer *);
+void		 session_close(struct peer *);
+void		 session_up(struct peer *);
+void		 session_down(struct peer *);
+void		 session_demote(struct peer *, int);
+void		 session_md5_reload(struct peer *);
 void		 session_stop(struct peer *, uint8_t, const char *);
 struct bgpd_addr *session_localaddr(struct peer *);
 
+/* session_bgp.c */
+void	session_open(struct peer *);
+void	session_keepalive(struct peer *);
+void	session_update(struct peer *, struct ibuf *);
+void	session_notification(struct peer *, uint8_t, uint8_t, struct ibuf *);
+void	session_notification_data(struct peer *, uint8_t, uint8_t, void *,
+	    size_t);
+void	session_rrefresh(struct peer *, uint8_t, uint8_t);
+int	session_dispatch_msg(struct pollfd *, struct peer *);
+void	session_process_msg(struct peer *);
+
+struct ibuf	*parse_header(struct ibuf *, void *, int *);
+
+void	start_timer_sendholdtime(struct peer *);
+void	bgp_fsm(struct peer *, enum session_events, struct ibuf *);
+void    change_state(struct peer *, enum session_state, enum session_events);
+
 /* timer.c */
 struct timer	*timer_get(struct timer_head *, enum Timer);
-struct timer	*timer_nextisdue(struct timer_head *, time_t);
-time_t		 timer_nextduein(struct timer_head *, time_t);
-int		 timer_running(struct timer_head *, enum Timer, time_t *);
+struct timer	*timer_nextisdue(struct timer_head *, monotime_t);
+monotime_t	 timer_nextduein(struct timer_head *);
+int		 timer_running(struct timer_head *, enum Timer, monotime_t *);
 void		 timer_set(struct timer_head *, enum Timer, u_int);
 void		 timer_stop(struct timer_head *, enum Timer);
 void		 timer_remove(struct timer_head *, enum Timer);
